
-module (resource_discovery_SUITE).

%% Behavioural Exports
-export ([
	  all/1,
	  init_per_testcase/2,
	  fin_per_testcase/2
         ]).

%% Test Case Exports
-export ([
          start_fixture/1, 
          test_process_tracking/1,
          stop_fixture/1
         ]).

-define (MATCH (X, Y), {?MODULE, ?LINE, X} = {?MODULE, ?LINE, Y}).

%% @doc top level test case.
all (doc)   -> ["Common Resource Manager functionality"];
all (suite) -> [
                {conf, start_fixture, [
                                       test_process_tracking
                                      ],
                 stop_fixture}
                ].

%% @doc called before each test case
init_per_testcase (Case, Config) -> Config.

%% @doc called after each test case
fin_per_testcase (Case, Config) -> ok.


%% @doc Start the fixture.
start_fixture(doc) -> ["start the fixture"];
start_fixture(suite) -> [];
start_fixture(Config) ->
    application:start(resource_discovery),
    Config.
     
%% @doc Stop the fixture.
stop_fixture(doc) -> ["stop the fixture"];
stop_fixture(suite) -> [];
stop_fixture(Config) ->
    Config.


%% @doc Test the remote api.
test_process_tracking(doc)    -> ["Test the ability to track process based resources.."];
test_process_tracking(suite)  -> [];
test_process_tracking(Config) -> 
    OtpDir = fs_test_lib:otp_dir(Config),
    resource_discovery:add_managed_targettypes([process, other_process]),

    Process = spawn(fun() -> process_resource() end),
    
    ?MATCH(true, fs_time:poll_until(fun() -> {ok, Process} == resource_discovery:get_resource(process) end, 10, 250)),
    ?MATCH(true, fs_time:poll_until(fun() -> {ok, Process} == resource_discovery:get_resource(other_process) end, 10, 250)),
    Process ! die,
    ?MATCH(true, fs_time:poll_until(fun() -> {error, no_resources} == resource_discovery:get_resource(process) end, 10, 250)),
    ?MATCH({error, no_resources}, resource_discovery:get_resource(other_process)),

    Config.

process_resource() ->
    %%Do this to test dual type membership for an instance.
    resource_discovery:add_localtypes([{process, self()}, {other_process, self()}]),
    resource_discovery:sync_inform_network(),

    %% Double registration single monitoring test
    resource_discovery:add_localtypes([{process, self()}, {other_process, self()}]),
    resource_discovery:sync_inform_network(),
    receive die -> ok end.
